
/*
** Grabs the name and email address from a From: header.
** This could get tricky; I've tried to keep it simple.
** Should be able to handle all the addresses below:
**
**   From: user                   [no @]
**   From: kent (Kent Landfield)  [no @ - with comment]
**   From: <user@node.domain>     [no text name, use email as text name]
**   From: Kent Landfield <kent>  [text name but no @]
**   From: (kent)                 [comment - no email address]
**   From: "" <kent>              [email address but null comment]
**   From:                        [blank From: line]
**   From: uu.net!kent            [uucp addresses - no comment]
**   From: uu.net!kent (kent)     [uucp addresses - with comment]
**   From: "(Joe Bloggs)" <joe@anorg.com> 
**   From: "Roy T. Fielding" <fielding@kiwi.ics.uci.edu>
**   From: kent@localhost
**   From: kent@uu.net (Kent Landfield)
**   From: (George Burgyan) <gburgyan@cybercon.com>
**   From: <gburgyan@cybercon.com> (George Burgyan) 
**   From:              Kent B. Landfield <kent@landfield.com>
**   From:      IN%"fekete+reply@c2.net" 26-JAN-1997 13:28:55.36
**   From:      IN%"vicric@panix.com"  "Vicki Richman" 13-AUG-1996 10:54:33.38
**   From:      US2RMC::"lwv26@cas.org" "Larry W. Virden, x2487" 22-OCT-1994 09:44:21.44
**   From:          Mail Delivery Subsystem <postmaster@igc.apc.org>
**   From:          Self <ehasbrouck>
**   From:         adam@eden.apana.org.au (Adam Frey)
**   From:        faqserv@penguin-lust.mit.edu
**   From:    nc0548@freebsd.netcom.com (Mark Hittinger)
**   From: "- Pam Greene, one of the *.answers moderators" <pgreene@MIT.EDU>
**   From: "Felan shena Thoron'edras" <felan@netcom.com>
**   From: David Muir Sharnoff <muir@idiom.com>
**   From: A.J.Doherty@reading.ac.uk (Andy Doherty)
**   From: Jordan Hubbard                        <jkh@vector.eikon.e-technik.tu-muenchen.de>
**   From: ZXPAN%SLACVM.BITNET@MITVMA.MIT.EDU
**   From: afs!piz!alf@uu5.psi.com (Alf the Poet)
**   From: answers@cp.tn.tudelft.nl ("Moderator *.answers")
**   From: mdw%merengue@merengue.oit.unc.edu (Matt Welsh)
**   From: bgoffe@whale.st.usm.edu (William L. Goffe)
**   From: " Zeev Meloch - email" <zeevm@email.foobar.co.il>
*
* 
* This is an interesting new one (1998-11-26):
* From: <name.hidden@era.ericsson.se>Name.Hidden@era.ericsson.se
*
*/

/****************************************************************************
 *
 * Getting name and email part from a From: -line
 *
 * Daniel Stenberg rewrote the whole routine Nov 24 1998.
 * Rewrote it again spring 1999.
 *
 * "This works even better"
 *
 ***************************************************************************/

#include "hypermail.h"
#include <stdio.h>
#include <string.h>

#include "setup.h"

void getname(char *buffer,
	     char **namep,
	     char **emailp)
{
  char *ptr;
  char *see;

  char *startofemail;
  char *endofemail;

  char *startofcomment;
  char *endofcomment;

  int emaillen;
  int commentlen;

  struct Push namebuf;
  struct Push emailbuf;

  /* start with NULLs */
  *namep = *emailp = NULL;

  /* first, init both string buffers */
  INIT_PUSH(namebuf);
  INIT_PUSH(emailbuf);

  ptr = buffer;

  if(!strncasecmp("From: ", ptr, 6)) {
    ptr+= 6;

    while(ptr && *ptr && isspace(*ptr))
      ptr++;
#ifdef DEBUG_GETFROM
    printf("** IN: %s", ptr);
#endif

    /* <one machine-usable reference> */
    /* (comment) */

    endofemail = NULL;
    startofemail=strchr(ptr, '<');
    if(startofemail) {
      endofemail=strchr(++startofemail, '>');
      emaillen = endofemail-startofemail;
#ifdef DEBUG_GETFROM
      printf("FOUND <>\n");
#endif
    }
    endofcomment = NULL;

    if(!endofemail) {
      startofcomment=strchr(ptr, '(');
      if(startofcomment) {
        endofcomment=strchr(++startofcomment, ')');
        commentlen=endofcomment-startofcomment;
#ifdef DEBUG_GETFROM
        printf("FOUND ()\n");
#endif
      }
    }

    /******** We found neither an email field nor comment ******/
    if(!endofemail && !endofcomment) {
      char at=0;
      /* We make an attempt to get an email address from the string! */
      startofemail = ptr;
      see = ptr;

      while(*see) {
        if('@' == *see) {
          at = 1;
        }
        else if(isspace(*see)) {
          if(at) {
            break;
          }
          if('\n' == *see)
            break;
          at = 0;
          startofemail = see+1;
        }
        see++;
      }
      if(at) {
        endofemail = see;
        emaillen = endofemail-startofemail;
#ifdef DEBUG_GETFROM
        printf("USE word with @!\n");
#endif
      }
      else {
        /* No, then the whole line is the email address! */
        endofemail = see;
        emaillen = endofemail-startofemail;
#ifdef DEBUG_GETFROM
        printf("USE last word without space!\n");          
#endif
      }

    }

    /******** We got an email field but no comment ******/
    if(!endofcomment && endofemail) {
      if(startofemail>ptr) {
        /* we set the text before the email address to be a comment */
        startofcomment = ptr;
        endofcomment = startofemail;
#ifdef DEBUG_GETFROM
        printf("USE comment BEFORE email\n");
#endif
      }
      else {
        /* we set the text after the email address to be a comment */
        startofcomment = endofemail+1;
        endofcomment = strchr(endofemail, '\n');
#ifdef DEBUG_GETFROM
        printf("USE comment AFTER email\n");
#endif
      }
      commentlen=endofcomment-startofcomment-1;
    }

    /******** We got a comment field but no email ******/
    if(endofcomment && !endofemail) {
      if(startofcomment>ptr) {
        /* we set the text before the comment to be the email */
        startofemail = ptr;
        endofemail = startofcomment;
#ifdef DEBUG_GETFROM
        printf("USE email BEFORE comment\n");
#endif
      }
      else {
        /* we set the text after the comment to be the email */
        startofemail = endofcomment+1;
        endofemail = strchr(endofcomment, '\n');
#ifdef DEBUG_GETFROM
        printf("USE email AFTER comment\n");
#endif
      }
      emaillen=endofemail-startofemail-1;
    }

    if(endofemail) {
      /* this may be an email address */
#ifdef DEBUG_GETFROM
      printf("EMAIL: %.*s\n", emaillen, startofemail);
#endif
      PushNString(&emailbuf, startofemail, emaillen);      

      /* If there's no @-letter, we may add a domain */
      if (use_domainaddr && 
          !strchr(PUSH_STRING(emailbuf), '@') ) { 
        /* don't modify uucp addresses */
        if (strchr(PUSH_STRING(emailbuf), '!')) {
          PushByte(&emailbuf, '@');
          PushString(&emailbuf, set_domainaddr);
        }
      }
    }
    if(endofcomment) {
      /* this may be an email address */
#ifdef DEBUG_GETFROM
      printf("COMMENT: %.*s\n", commentlen, startofcomment);
#endif

      /* pass all initiating white spaces or quotes */
      while(*startofcomment &&
            (isspace(*startofcomment) || *startofcomment=='\"')) {
        startofcomment++;
        commentlen--;
      }

      /* don't count the trailing white spaces or quotes */
      while(commentlen &&
            (isspace(startofcomment[commentlen-1]) ||
             startofcomment[commentlen-1] == '\"')) {
        commentlen--;
      }
      PushNString(&namebuf, startofcomment, commentlen);      
    }

    if(!endofemail && !endofcomment) {
#ifdef DEBUG_GETFROM
      printf("-- FAILED: %s", ptr);
#endif
    }
  }
  *namep = PUSH_STRING(namebuf);
  *emailp = PUSH_STRING(emailbuf);
}


#if 0 /* this is the previous one, for information only */

struct Part822 {
  struct Push buf;
  int type;
  int count; /* 0 if its the first, 1 if its the second */
};

/* we *OR* these flags! */

#define RFC822_EMAIL 1
#define RFC822_NAME  2

#define RFC822_GUESS 4 /* this is just guessed, a non-guessed field will
			  have priority over a guessed one */

#define RFC822_PARTS (RFC822_EMAIL|RFC822_NAME)

void getname(char *in,
	     char **namep,
	     char **emailp)
{
  char quotemode=FALSE;
  struct Part822 part822[2];
  int bufcount=0;
  char done=FALSE;
  int ws;
  char emailbuffer[256];
  int prevpart = 0;

  struct Push *namebufp;
  struct Push *emailbufp;

  struct Part822 *bufp = &part822[0];

  /* start with NULLs */
  *namep = *emailp = NULL;

  if(strncasecmp("From: ", in, 6))
    return; /* not a valid from line */

  /* first, init both string buffers */
  INIT_PUSH(part822[0].buf);
  INIT_PUSH(part822[1].buf);

  in += 6; /* pass the initiator */

  while (isspace(*in))
    in++; /* pass beginning whitespace */

  if(!*in) {
    /* empty line, abort */
    PushString(&part822[0].buf, NONAME);
    *namep=PUSH_STRING(part822[0].buf);

    PushString(&part822[1].buf, NOEMAIL);
    *emailp=PUSH_STRING(part822[1].buf);
    return;
  }

  bufp->type = 0;
  bufp->count = 0;

  while(!done) {
    if(*in == '\"') {
      quotemode^=TRUE;
      in++;
    }

    switch(*in) {
    case '\0':
    case '\n':
    case '\r':
      done = TRUE;
      continue;
    case '<':
      if(bufp->buf.len) {
        /* We already have filled in a buffer, and then we take this
           < as a mark to start a new one: */
        bufcount++;
        if(2 == bufcount) {
          /* this is getting silly */
          done = TRUE;
          continue;
        }
        prevpart = bufp->type; /* what kind of part the previous one was */
        bufp++;
        bufp->type = 0;
        bufp->count = bufcount;
        /* advance to the next buffer to fill in */
      }
      /* we don't expect any quotes here */
      sscanf(in+1, "%255[^>]", emailbuffer);
      in += strlen(emailbuffer)+2;
      PushString(&bufp->buf, emailbuffer);
      bufp->type = RFC822_EMAIL;

      if(prevpart & RFC822_EMAIL) {
	/* the previous part is ALSO said to be the email part, we
	   don't believe there is any better way than this to find an
	   email address, so we *convert* the previous one to name part!
	   */
	part822[0].type = RFC822_NAME|RFC822_GUESS;
      }

      /* ok, we've seen cases where the Real Name follows this part
       * _without_ whitespaces after this, so we need to expect
       * another part already now */

      /* FALL THROUGH! */

    case '\t':
    case ' ':
      while (isspace(*in))
	in++; /* pass more whitespaces */
      bufcount++;
      if(2 == bufcount) {
	/* this is getting silly */
	done = TRUE;
	continue;
      }
      prevpart = bufp->type; /* what kind of part the previous one was */
      bufp++;
      bufp->type = 0;
      bufp->count = bufcount;
      break;

    case '(':
      in++;
      while(*in) {
	if(quotemode && (*in == '\\')) {
	  in++;
	  PushByte(&bufp->buf, *in++);
	}
	else {
	  if(!*in || (!quotemode && (*in == ')')))
	    break;
	  else if(*in == '\"') {
	    quotemode ^= TRUE; /* toggle mode */
	    in++; /* pass the quote character */
	  }
	  else
	    PushByte(&bufp->buf, *in++);
	}
      }
      if(*in == ')')
	in++;
      bufp->type = RFC822_NAME;
      break;
    default:
      ws=0;
      while(*in) {
	if(quotemode && (*in == '\\')) {
	  in++;
	  PushByte(&bufp->buf, *in++);
	}
	else {
	  if(!*in || 
	     /* lessthans aren't important when quoting! */

             /* removed sensitive for open parens 1998-12-10 */
	     (!quotemode && (*in == '<')))  {
	    if(ws)
	      in--; /* back on the whitespace */
	    break;
	  }
	  else if(*in == '\"') {
	    quotemode ^= TRUE; /* toggle mode */
	    ws=0;
	    in++; /* pass the quote character */
	  }
	  else if(isspace(*in)) {
	    if(!quotemode && (bufp->type&RFC822_EMAIL)) {
	      /* we have previously detected this to be an email address,
		 and therefore this cannot have spaces */
	      break;
	    }
	    ws++;
	    in++;
	  }
	  else {
	    if(ws) {
	      PushByte(&bufp->buf, ' ');
	      ws=0;
	    }
	    if( ('@' == *in)  &&
		PUSH_STRLEN(bufp->buf) && /* we can't start with an @ sign */
		!(prevpart & RFC822_EMAIL) ) {
	      char *emp;
	      /* We treat a @-letter here as a hint of this being an
                 email address! But _not_ if we already have found an email
		 in the previous part */
	      
	      /* Have we already found an @-sign in this part earlier? */
	      if(! strchr( PUSH_STRING(bufp->buf), '@')) {

		/* Email addresses can not have 8bit characters, we check that
		   to be a little more certain! */
		emp = PUSH_STRING(bufp->buf);
		
		while(*emp) {
		  if((unsigned char)*emp > 127)
		    break;
		  emp++;
		}
		if(!*emp)
		  bufp->type = RFC822_EMAIL|RFC822_GUESS;
	      }
	      else
		/* can't have to @s */
		bufp->type &= ~RFC822_EMAIL;
	    }
	    PushByte(&bufp->buf, *in++);
	  }
	}
      }
      break;
    }
  }
  if(0 == bufcount) {
    switch(part822[0].type & RFC822_PARTS) {
    case RFC822_NAME:
      namebufp = &part822[0].buf;

      PushString(&part822[1].buf, NOEMAIL);
      emailbufp = &part822[1].buf;

      /* We return here to prevent the noemail message to get domainized! */
      *namep = PUSH_STRING(*namebufp);
      *emailp = PUSH_STRING(*emailbufp);
      return;

    case RFC822_EMAIL:
    default:
      emailbufp = &part822[0].buf;

      /* we have no name, so we set the email to be the name too */
      PushString(&part822[1].buf, PUSH_STRING(part822[0].buf));
      namebufp = &part822[1].buf;
      break;
    }
  }
  else if((part822[0].type & RFC822_NAME) ||
	  (part822[1].type & RFC822_EMAIL)) {
      namebufp = &part822[0].buf;
      emailbufp = &part822[1].buf;
  }
  else {
    namebufp = &part822[1].buf;
    emailbufp = &part822[0].buf;
  }

  /* now, do the email magic checks */
  if (use_domainaddr && 
      !strchr(PUSH_STRING(*emailbufp), '@') ) { 
    /* don't modify uucp addresses */
    if (strchr(PUSH_STRING(*emailbufp), '!')) {
      PushByte(emailbufp, '@');
      PushString(emailbufp, set_domainaddr);
    }
  }
  *namep = PUSH_STRING(*namebufp);
  *emailp = PUSH_STRING(*emailbufp);
}

#endif
